@@ -16,17 +16,24 @@ class Puppet::Node::Facts::Puppetdb < Puppet::Indirector::REST
 
   def find(request)
     begin
-      response = http_get(request, "/v2/#{request.key}/facts", headers)
+      response = http_get(request, "/v2/nodes/#{CGI.escape(request.key)}/facts", headers)
       log_x_deprecation_header(response)
 
       if response.is_a? Net::HTTPSuccess
         result = PSON.parse(response.body)
+        # Note: the Inventory Service API appears to expect us to return nil here
+        # if the node isn't found.  However, PuppetDB returns an empty array in
+        # this case; for now we will just look for that condition and assume that
+        # it means that the node wasn't found, so we will return nil.  In the
+        # future we may want to improve the logic such that we can distinguish
+        # between the "node not found" and the "no facts for this node" cases.
+        if result.empty?
+          return nil
+        end
         facts = result.inject({}) do |a,h|
           a.merge(h['name'] => h['value'])
         end
         Puppet::Node::Facts.new(request.key, facts)
-      elsif response.is_a? Net::HTTPNotFound
-        nil
       else
         # Newline characters cause an HTTP error, so strip them
         raise "[#{response.code} #{response.message}] #{response.body.gsub(/[\r\n]/, '')}"
