@@ -5,25 +5,17 @@
 (def version-string
   (memoize
    (fn []
-     "Determine the version number using 'git describe'"
+     "Determine the version number using 'rake version -s'"
      []
      (if (.exists (file "version"))
        (s/trim (slurp "version"))
-       (let [command                ["git" "describe"]
+       (let [command                ["rake" "version" "-s"]
              {:keys [exit out err]} (apply sh command)]
          (when-not (zero? exit)
            (println (format "Non-zero exit status during version check:\n%s\n%s\n%s\n%s"
                             command exit out err))
            (System/exit 1))
-
-         ;; We just want the first 4 "components" of the version string,
-         ;; joined with dots
-         (-> out
-           (s/trim)
-           (s/replace #"-" ".")
-           (s/split #"\.")
-           (#(take 4 %))
-           (#(s/join "." %))))))))
+       (s/trim out))))))
 
 (defproject puppetdb (version-string)
   :description "Puppet-integrated catalog and fact storage"
@@ -49,6 +41,8 @@
                  [org.ini4j/ini4j "0.5.2"]
                  ;; Version information
                  [trptcolin/versioneer "0.1.0"]
+                 ;; Job scheduling
+                 [overtone/at-at "1.1.1"]
                  ;; Nicer exception handling with try+/throw+
                  [slingshot "0.10.3"]
                  [digest "1.3.0"]
@@ -70,7 +64,8 @@
                  [net.cgrand/moustache "1.1.0" :exclusions [ring/ring-core org.clojure/clojure]]
                  [clj-http "0.5.3"]
                  [ring/ring-core "1.1.1"]
-                 [ring/ring-jetty-adapter "1.1.1"]]
+                 [ring/ring-jetty-adapter "1.1.1"]
+                 [org.apache.commons/commons-compress "1.4.1"]]
 
   :profiles {:dev {:resource-paths ["test-resources"],
                    :dependencies [[ring-mock "0.1.1"]]}}
